import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_location/ocg_location.dart';
import 'package:ocg_location/src/repository/location.dart';

///
/// Events
///
abstract class LocationCurrentEvent {}

class LocationCurrentRefreshEvent extends LocationCurrentEvent {
  final bool canLoadDetails;

  LocationCurrentRefreshEvent({this.canLoadDetails = false});
}

abstract class LocationCurrentState {}

class LocationCurrentIdleState extends LocationCurrentState {}

class LocationCurrentLoadingState extends LocationCurrentState {}

class LocationCurrentLoadedState extends LocationCurrentState {
  final Address currentLocation;
  LocationCurrentLoadedState(this.currentLocation);
}

class LocationCurrentErrorState extends LocationCurrentState {
  final String error;

  LocationCurrentErrorState(this.error);
}

class LocationCurrentBloc
    extends Bloc<LocationCurrentEvent, LocationCurrentState> {
  final LocationRepository locationRepository;
  LocationCurrentBloc(this.locationRepository)
      : super(LocationCurrentIdleState());

  @override
  Stream<LocationCurrentState> mapEventToState(
      LocationCurrentEvent event) async* {
    if (event is LocationCurrentRefreshEvent) {
      yield LocationCurrentLoadingState();
      AppResponse<Position> response =
          await locationRepository.getCurrentLocation();
      if (response.isSuccess) {
        Position latLng =
            Position(response.data.latitude, response.data.longitude);
        if (event.canLoadDetails) {
          AppResponse<Address> addressResponse =
              await locationRepository.getLocationDetailsByPosition(latLng);
          if (addressResponse.isSuccess)
            yield LocationCurrentLoadedState(addressResponse.data);
          else {
            yield LocationCurrentLoadedState(Address.named(
                latitude: latLng.latitude, longitude: latLng.longitude));
          }
        } else {
          yield LocationCurrentLoadedState(Address.named(
              latitude: latLng.latitude, longitude: latLng.longitude));
        }
      } else {
        yield LocationCurrentErrorState(response.error);
      }
    }
  }
}
