import 'package:json_annotation/json_annotation.dart';
import 'package:ocg_location/ocg_location.dart';

part 'address.g.dart';

@JsonSerializable(explicitToJson: true)
class Address {
  String fullAddress;
  String area;
  String pinCode;
  String country;
  String placeId;
  String locality;
  double latitude;
  double longitude;

  Address(this.fullAddress, this.area, this.pinCode, this.country, this.placeId,
      this.locality, this.latitude, this.longitude);

  Address.named(
      {this.fullAddress,
      this.area,
      this.pinCode,
      this.country,
      this.placeId,
      this.locality,
      this.latitude,
      this.longitude});

  Map<String, dynamic> toJson() {
    var json = _$AddressToJson(this);
    json["geohash"] = geohash;
    return json;
  }

  factory Address.fromJson(json) => _$AddressFromJson(json);

  String get geohash {
    String hash = LocationRepository.geoHash(latitude, longitude);
    return hash;
  }

  Position get position =>
      Position.named(latitude: latitude, longitude: longitude);
}

class RangeGeoHash {
  final String lower;
  final String upper;

  RangeGeoHash(this.lower, this.upper);

  RangeGeoHash.named({this.lower, this.upper});
}
